@@ -8,7 +8,7 @@ use Encode      qw();
 use Carp        qw(croak);
 use base        'Exporter';
 
-our $VERSION   = '1.0.19';
+our $VERSION   = '1.0.20';
 our @EXPORT_OK = qw(markdown);
 
 =head1 NAME
@@ -930,8 +930,8 @@ sub _DoLists {
                 # paragraph for the last item in a list, if necessary:
                 $list =~ s/\n{2,}/\n\n\n/g;
                 my $result = ( $list_type eq 'ul' ) ?
-                    $self->_ProcessListItems($list, $marker_ul)
-                  : $self->_ProcessListItems($list, $marker_ol);
+                    $self->_ProcessListItemsUL($list, $marker_ul)
+                  : $self->_ProcessListItemsOL($list, $marker_ol);
                 $result = "<$list_type>\n" . $result . "</$list_type>\n";
                 $result;
             }egmx;
@@ -947,8 +947,8 @@ sub _DoLists {
                 # paragraph for the last item in a list, if necessary:
                 $list =~ s/\n{2,}/\n\n\n/g;
                 my $result = ( $list_type eq 'ul' ) ?
-                    $self->_ProcessListItems($list, $marker_ul)
-                  : $self->_ProcessListItems($list, $marker_ol);
+                    $self->_ProcessListItemsUL($list, $marker_ul)
+                  : $self->_ProcessListItemsOL($list, $marker_ol);
                 $result = "<$list_type>\n" . $result . "</$list_type>\n";
                 $result;
             }egmx;
@@ -958,9 +958,9 @@ sub _DoLists {
     return $text;
 }
 
-sub _ProcessListItems {
+sub _ProcessListItemsOL {
 #
-#   Process the contents of a single ordered or unordered list, splitting it
+#   Process the contents of a single ordered list, splitting it
 #   into individual list items.
 #
 
@@ -1017,7 +1017,72 @@ sub _ProcessListItems {
         }
 
         "<li>" . $item . "</li>\n";
-    }egmx;
+    }egmxo;
+
+    $self->{_list_level}--;
+    return $list_str;
+}
+
+sub _ProcessListItemsUL {
+#
+#   Process the contents of a single unordered list, splitting it
+#   into individual list items.
+#
+
+    my ($self, $list_str, $marker_any) = @_;
+
+
+    # The $self->{_list_level} global keeps track of when we're inside a list.
+    # Each time we enter a list, we increment it; when we leave a list,
+    # we decrement. If it's zero, we're not in a list anymore.
+    #
+    # We do this because when we're not inside a list, we want to treat
+    # something like this:
+    #
+    #       I recommend upgrading to version
+    #       8. Oops, now this line is treated
+    #       as a sub-list.
+    #
+    # As a single paragraph, despite the fact that the second line starts
+    # with a digit-period-space sequence.
+    #
+    # Whereas when we're inside a list (or sub-list), that line will be
+    # treated as the start of a sub-list. What a kludge, huh? This is
+    # an aspect of Markdown's syntax that's hard to parse perfectly
+    # without resorting to mind-reading. Perhaps the solution is to
+    # change the syntax rules such that sub-lists must start with a
+    # starting cardinal number; e.g. "1." or "a.".
+
+    $self->{_list_level}++;
+
+    # trim trailing blank lines:
+    $list_str =~ s/\n{2,}\z/\n/;
+
+
+    $list_str =~ s{
+        (\n)?                           # leading line = $1
+        (^[ \t]*)                       # leading whitespace = $2
+        ($marker_any) [ \t]+            # list marker = $3
+        ((?s:.+?)                       # list item text   = $4
+        (\n{1,2}))
+        (?= \n* (\z | \2 ($marker_any) [ \t]+))
+    }{
+        my $item = $4;
+        my $leading_line = $1;
+        my $leading_space = $2;
+
+        if ($leading_line or ($item =~ m/\n{2,}/)) {
+            $item = $self->_RunBlockGamut($self->_Outdent($item));
+        }
+        else {
+            # Recursion for sub-lists:
+            $item = $self->_DoLists($self->_Outdent($item));
+            chomp $item;
+            $item = $self->_RunSpanGamut($item);
+        }
+
+        "<li>" . $item . "</li>\n";
+    }egmxo;
 
     $self->{_list_level}--;
     return $list_str;
